float4x4 View;
float3 ViewPosition;
float4x4 Projection;
#define NUM_TEXTURES 8
#define MAX_OBJECTS 45
//The rotation matrix is also used independently to transform the normal.
//float4x4 WorldMatrices[MAX_OBJECTS];
extern float3x3 Rotations[MAX_OBJECTS];
extern float3 Translations[MAX_OBJECTS];
extern float TextureIndices[MAX_OBJECTS];

extern uniform texture Texture0;
extern uniform texture Texture1;
extern uniform texture Texture2;
extern uniform texture Texture3;
extern uniform texture Texture4;
extern uniform texture Texture5;
extern uniform texture Texture6;
extern uniform texture Texture7;

extern uniform float3 LightDirection;
extern uniform float4 DiffuseColor;
extern uniform float AmbientAmount;
extern uniform float4 SpecularColor;


sampler TextureSamplers[NUM_TEXTURES] = 
{

sampler_state
{
    Texture = (Texture0);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
},

sampler_state
{
    Texture = (Texture1);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
},

sampler_state
{
    Texture = (Texture2);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
},

sampler_state
{
    Texture = (Texture3);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
},

sampler_state
{
    Texture = (Texture4);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
},

sampler_state
{
    Texture = (Texture5);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
},

sampler_state
{
    Texture = (Texture6);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
},

sampler_state
{
    Texture = (Texture7);
    
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = Linear;
    
    AddressU = Wrap;
    AddressV = Wrap;
}

};


struct VertexShaderInput
{
    float4 Position : POSITION0;
    float3 Normal : TEXCOORD0;
    float2 TextureCoordinates : TEXCOORD1;
    float Index : TEXCOORD2;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
    float3 Normal : TEXCOORD0;
    float2 TextureCoordinates : TEXCOORD1;
    float Index : TEXCOORD2;
    float3 view :TEXCOORD3;
};



VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;
    //Set up the 4x4 expansion of the 3x3 rotation matrix to transform the position.  This could be optimized.
    int index = round(input.Index);
    float3x3 rot = Rotations[index];
    float4x4 localRot = {rot[0][0],rot[0][1],rot[0][2],0,
						rot[1][0],rot[1][1],rot[1][2],0,
						rot[2][0],rot[2][1],rot[2][2],0,
						0,0,0,1};
	float4 localTrans = {Translations[index].x, Translations[index].y,Translations[index].z,0};
    float4 worldPosition = mul(input.Position, localRot) + localTrans;
    output.Position = mul(mul(worldPosition, View), Projection);
    output.TextureCoordinates = input.TextureCoordinates;
    output.Normal = input.Normal;
    output.Index = input.Index;
    
    output.view = normalize(ViewPosition.xyz - worldPosition.xyz);
	

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
    float index = round(input.Index);
    int x = TextureIndices[index];
    float4 surfaceColor;
    if(x == 0)
		surfaceColor = tex2D(TextureSamplers[0], input.TextureCoordinates);
	else if(x == 1)
		surfaceColor = tex2D(TextureSamplers[1], input.TextureCoordinates);
	else if(x == 2)
		surfaceColor = tex2D(TextureSamplers[2], input.TextureCoordinates);
	else if(x == 3)
		surfaceColor = tex2D(TextureSamplers[3], input.TextureCoordinates);
	else if(x == 4)
		surfaceColor = tex2D(TextureSamplers[4], input.TextureCoordinates);
	else if(x == 5)
		surfaceColor = tex2D(TextureSamplers[5], input.TextureCoordinates);
	else if(x == 6)
		surfaceColor = tex2D(TextureSamplers[6], input.TextureCoordinates);
	else
		surfaceColor = tex2D(TextureSamplers[7], input.TextureCoordinates);
		
	float3 normal = mul(normalize(input.Normal), Rotations[index]);
    float diffuseAmount = saturate(-dot(normal, LightDirection));
    float3 halfVector = (LightDirection - input.view) / -2;

    
    float specular = pow( saturate( dot(normal, halfVector) ), 8);
    
	surfaceColor =  AmbientAmount * surfaceColor + surfaceColor * diffuseAmount * DiffuseColor + SpecularColor * specular;
    return surfaceColor;
}

technique Technique1
{
    pass Pass1
    {
        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }
}
