using System;
using System.Collections.Generic;
using System.Text;
using BEPUphysics;
using Microsoft.Xna.Framework;
using System.Diagnostics;
using BEPUphysics.Entities;
using BEPUphysics.Events;

namespace BEPUphysicsDemos
{
    /// <summary>
    /// Handles events for the Events demo (#30).
    /// </summary>
    class EntityEventListenerDemo30
    {
        Entity detector;
        Entity acceptedTrigger;
        Entity spawnedBox;

        /// <summary>
        /// Sets up the demos listener.
        /// </summary>
        /// <param name="detector">The entity which will be monitored for collisions.</param>
        /// <param name="acceptedTrigger">The entity which is accepted by the detector to trigger the event.</param>
        public EntityEventListenerDemo30(Entity detector, Entity acceptedTrigger)
        {
            this.detector = detector;
            this.acceptedTrigger = acceptedTrigger;
            spawnedBox = new Box(new Vector3(-4, 5, 0), 1, 1, 1, 1);
            detector.space.add(spawnedBox);
            detector.addEventHook(new EventHandlerInitialCollisionDetected(initialCollisionDetected));
            detector.addEventHook(new EventHandlerCollisionEnded(collisionEnded));

        }

        public void initialCollisionDetected(Entity sender, Controller controller)
        {
            if (controller.colliderA == acceptedTrigger || controller.colliderB == acceptedTrigger)
            {//If the detector collided with the accepted trigger, move the box.
                spawnedBox.centerPosition = new Vector3(4, 5, 0);
                spawnedBox.orientationQuaternion = Quaternion.Identity;
                spawnedBox.linearVelocity = Vector3.Zero;
                spawnedBox.angularVelocity = Vector3.Zero;
            }


        }

        public void collisionEnded(Entity sender, Controller controller)
        {

            if (controller.colliderA == acceptedTrigger || controller.colliderB == acceptedTrigger)
            {//If the detector ceases to collide, get rid of the spawned box.
                spawnedBox.centerPosition = new Vector3(-4, 5, 0);
                spawnedBox.orientationQuaternion = Quaternion.Identity;
                spawnedBox.linearVelocity = Vector3.Zero;
                spawnedBox.angularVelocity = Vector3.Zero;
            }

        }







    }

    /// <summary>
    /// Handles events for the Detector Volume demo (#32).
    /// As the entity enters and exits the concave detector volume, the displayBox's color changes.
    /// </summary>
    public class DetectorVolumeEventListenerDemo32
    {
        DisplayBox displayBox;

        public DetectorVolumeEventListenerDemo32(DetectorVolume detectorVolume, DisplayBox displayBox)
        {
            this.displayBox = displayBox;

            detectorVolume.eventEntityBeginsTouchingVolume += new EventHandlerEntityBeginsTouchingVolume(detectorVolume_eventEntityBeginsTouchingVolume);
            detectorVolume.eventEntityStopsTouchingVolume += new EventHandlerEntityStopsTouchingVolume(detectorVolume_eventEntityStopsTouchingVolume);
            detectorVolume.eventVolumeBeginsContainingEntity += new EventHandlerVolumeBeginsContainingEntity(detectorVolume_eventVolumeBeginsContainingEntity);
            detectorVolume.eventVolumeStopsContainingEntity += new EventHandlerVolumeStopsContainingEntity(detectorVolume_eventVolumeStopsContainingEntity);

                   
        }

        void detectorVolume_eventVolumeStopsContainingEntity(DetectorVolume volume, Entity entity)
        {
            displayBox.textureIndex = 3;
        }

        void detectorVolume_eventVolumeBeginsContainingEntity(DetectorVolume volume, Entity entity)
        {
            displayBox.textureIndex = 2;
        }

        void detectorVolume_eventEntityStopsTouchingVolume(Entity toucher, DetectorVolume volume)
        {
            displayBox.textureIndex = 0;
        }

        void detectorVolume_eventEntityBeginsTouchingVolume(Entity toucher, DetectorVolume volume)
        {
            displayBox.textureIndex = 1;
        }
    }
}
